//
//  TableViewController.m
//  Insert
//
//  Created by nyuguest on 8/14/13.
//  Copyright (c) 2013 nyuguest. All rights reserved.
//

#import "TableViewController.h"

@interface TableViewController ()
@end

@implementation TableViewController

- (void) setTitle {
	self.title = [NSString stringWithFormat:
		@"TableView with %u Cell%@",
		lines.count,
		lines.count == 1 ? @"" : @"s"
	];
}


- (id) initWithStyle: (UITableViewStyle) style
{
	self = [super initWithStyle: style];
	if (self) {
		// Custom initialization
		cellReuseIdentifier = @"insert";

		lines = [NSMutableArray arrayWithObjects:
			@"First line",
			@"Second line",
			@"Third line",
			nil
		];

		[self setTitle];
	}
	return self;
}

- (void) viewDidLoad
{
	[super viewDidLoad];

	[self.tableView registerClass: [UITableViewCell class]
		forCellReuseIdentifier: cellReuseIdentifier];

	// Uncomment the following line to preserve selection between presentations.
	// self.clearsSelectionOnViewWillAppear = NO;

	// Uncomment the following line to display an Edit button in the navigation bar for this view controller.
	self.navigationItem.rightBarButtonItem = self.editButtonItem;
}

- (void) didReceiveMemoryWarning
{
	[super didReceiveMemoryWarning];
	// Dispose of any resources that can be recreated.
}

#pragma mark - Table view data source

- (NSInteger) numberOfSectionsInTableView: (UITableView *) tableView
{
	// Return the number of sections.
	return 1;
}

- (NSInteger) tableView: (UITableView *) tableView numberOfRowsInSection: (NSInteger) section
{
	// Return the number of rows in the section.
	return lines.count;
}

- (UITableViewCell *) tableView: (UITableView *) tableView cellForRowAtIndexPath: (NSIndexPath *) indexPath
{
	UITableViewCell *cell = [tableView dequeueReusableCellWithIdentifier: cellReuseIdentifier forIndexPath: indexPath];

	// Configure the cell...
	cell.textLabel.text = [lines objectAtIndex: indexPath.row];
	return cell;
}

/*
// Override to support conditional editing of the table view.
- (BOOL) tableView: (UITableView *) tableView canEditRowAtIndexPath: (NSIndexPath *) indexPath
{
	// Return NO if you do not want the specified item to be editable.
	return YES;
}
*/


// Override to support editing the table view.
- (void) tableView: (UITableView *) tableView commitEditingStyle: (UITableViewCellEditingStyle) editingStyle forRowAtIndexPath: (NSIndexPath *) indexPath
{
	if (editingStyle == UITableViewCellEditingStyleDelete) {
		// Delete the row from the data source
		[lines removeObjectAtIndex: indexPath.row];
		
		NSArray *indexPaths = [NSArray arrayWithObject: indexPath];
		[tableView deleteRowsAtIndexPaths: indexPaths withRowAnimation: UITableViewRowAnimationFade];
	}
	
	else if (editingStyle == UITableViewCellEditingStyleInsert && changing == nil) {
		// Create a new instance of the appropriate class, insert it into the array,
		// and add a new row to the table view.
		[lines addObject: @"New line"];
		[self.tableView reloadData];

		//The new cell's textLabel has an all-zero frame.
		//We will copy the frame of an established cell instead.
		NSIndexPath *ip = [NSIndexPath indexPathForRow: 0 inSection: 0];
		UITableViewCell *archetype = [tableView cellForRowAtIndexPath: ip];
		
		//Edit the text of the new cell.
		changing = [NSIndexPath indexPathForRow: lines.count - 1 inSection: 0];

		UITableViewCell *cell = [tableView cellForRowAtIndexPath: changing];
		UIFont *font = archetype.textLabel.font;		
		CGFloat dy = (cell.contentView.bounds.size.height - font.lineHeight) / 2;
		
		CGRect frame = CGRectMake(
			archetype.textLabel.frame.origin.x,
			archetype.textLabel.frame.origin.y + dy,
			archetype.contentView.bounds.size.width,
			archetype.contentView.bounds.size.height - dy
		);
		
		UITextField *textField = [[UITextField alloc] initWithFrame: frame];
		textField.delegate = self;
		textField.backgroundColor = cell.textLabel.backgroundColor;
		textField.textColor = cell.textLabel.textColor;
		textField.font = font;
		textField.text = cell.textLabel.text;

		cell.textLabel.text = @" ";
		[cell.contentView addSubview: textField];
		[textField becomeFirstResponder];
		[tableView deselectRowAtIndexPath: indexPath animated: YES];
    }
	
	[self setTitle]; 
}


// Override to support rearranging the table view.
- (void) tableView: (UITableView *) tableView moveRowAtIndexPath: (NSIndexPath *) fromIndexPath toIndexPath: (NSIndexPath *)toIndexPath
{
	NSString *line = [lines objectAtIndex: fromIndexPath.row];
	[lines removeObjectAtIndex: fromIndexPath.row];
	[lines insertObject: line atIndex: toIndexPath.row];
	
	if (changing.row == fromIndexPath.row) {
		changing = [toIndexPath copy];
	}
}


// Override to support conditional rearranging of the table view.
- (BOOL) tableView: (UITableView *) tableView canMoveRowAtIndexPath: (NSIndexPath *)indexPath
{
	// Return NO if you do not want the item to be re-orderable.
	return YES;
}


#pragma mark - Table view delegate

//Give the last (bottommost) cell a green plus.  Give the others red minuses.

- (UITableViewCellEditingStyle) tableView: (UITableView *) tableView
	editingStyleForRowAtIndexPath: (NSIndexPath *) indexPath
{
	if (indexPath.row == lines.count - 1) {
		return UITableViewCellEditingStyleInsert;
	}
	return UITableViewCellEditingStyleDelete;
}


- (void) tableView: (UITableView *) tableView didSelectRowAtIndexPath: (NSIndexPath *) indexPath
{
	// Navigation logic may go here. Create and push another view controller.
	/*
	<#DetailViewController#> *detailViewController = [[<#DetailViewController#> alloc] initWithNibName: @"<#Nib name#>" bundle: nil];
	// ...
	// Pass the selected object to the new view controller.
	[self.navigationController pushViewController:detailViewController animated:YES];
	*/
}

#pragma mark -
#pragma mark protocol UITextFieldDelegate

//Called when return key is pressed.
//Decide if the app should accept this input and hide the keyboard.

- (BOOL) textFieldShouldReturn: (UITextField *) textField {
	[textField resignFirstResponder];	//Hide keyboard.
	return YES;
}

//Called when keyboard is hidden.
//Process the text that was input.

- (void) textFieldDidEndEditing: (UITextField *) textField {
	//Get the cell that was being edited.
	UITableViewCell *cell = [self.tableView cellForRowAtIndexPath: changing];
	
	[lines removeObjectAtIndex: changing.row];
	[lines insertObject: textField.text atIndex: changing.row];
	cell.textLabel.text = textField.text;
	[textField removeFromSuperview];
	changing = nil;
}

@end
